# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.admin.helpers import AdminReadonlyField
from django.core.exceptions import ObjectDoesNotExist
from django.db.models.fields.related import ManyToManyRel
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe

from ._compat import lookup_field, smart_unicode


def get_empty_value_display(cls):
    if hasattr(cls.model_admin, 'get_empty_value_display'):
        return cls.model_admin.get_empty_value_display()
    else:
        # Django < 1.9
        from django.contrib.admin.views.main import EMPTY_CHANGELIST_VALUE
        return EMPTY_CHANGELIST_VALUE


def djmoney_contents(self):
    from django.contrib.admin.templatetags.admin_list import _boolean_icon

    field, obj, model_admin = self.field['field'], self.form.instance, self.model_admin

    try:
        f, attr, value = lookup_field(field, obj, model_admin)
    except (AttributeError, ValueError, ObjectDoesNotExist):
        result_repr = get_empty_value_display(self)
    else:
        if f is None:
            boolean = getattr(attr, 'boolean', False)
            if boolean:
                result_repr = _boolean_icon(value)
            else:
                result_repr = smart_unicode(value)
                if getattr(attr, 'allow_tags', False):
                    result_repr = mark_safe(result_repr)
        else:
            if value is None:
                result_repr = get_empty_value_display(self)
            elif isinstance(f.rel, ManyToManyRel):
                result_repr = ', '.join(map(str, value.all()))
            else:
                result_repr = smart_unicode(value)
    return conditional_escape(result_repr)


AdminReadonlyField.contents = djmoney_contents



# -*- coding: utf-8 -*-
# flake8: noqa
from django import VERSION


try:
    from django.db.models.constants import LOOKUP_SEP
except ImportError:
    # Django < 1.5
    LOOKUP_SEP = '__'

try:
    from django.db.models.expressions import BaseExpression
except ImportError:
    # Django < 1.8
    from django.db.models.expressions import ExpressionNode as BaseExpression

try:
    from django.db.models.expressions import Expression
except ImportError:
    # Django < 1.8
    from django.db.models.sql.expressions import SQLEvaluator as Expression

try:
    from django.contrib.admin.utils import lookup_field
except ImportError:
    from django.contrib.admin.util import lookup_field

try:
    from django.utils.encoding import smart_unicode
except ImportError:
    # Python 3
    from django.utils.encoding import smart_text as smart_unicode

try:
    string_types = (basestring,)
except NameError:
    string_types = (str, bytes)


if VERSION >= (1, 7):
    from django.utils.deconstruct import deconstructible
else:
    def deconstructible(cls):
        return cls


def split_expression(expr):
    """
    Returns lhs and rhs of the expression.
    """
    if VERSION < (1, 8):
        return expr.children
    else:
        return expr.lhs, expr.rhs


def set_expression_rhs(expr, value):
    """
    Sets right hand side value of the expression.
    """
    if VERSION < (1, 8):
        expr.children[1] = value
    else:
        expr.rhs.value = value


def get_fields(model):
    """
    Returns a set of fields associated to the model.
    """
    opts = model._meta
    if VERSION < (1, 8):
        return opts.get_all_field_names()
    else:
        return set(field.name for field in opts.get_fields())
